/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is infoRSS.
 *
 * The Initial Developer of the Original Code is
 *   Didier Ernotte <didier@ernotte.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Didier Ernotte <didier@ernotte.com>.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
//------------------------------------------------------------------------------
// inforssFeedNntp
// Author : Didier Ernotte 2005
// Inforss extension
//------------------------------------------------------------------------------
var inforss = inforss || {};
Components.utils.import("chrome://inforss/content/modules/Debug.jsm", inforss);
Components.utils.import("chrome://inforss/content/modules/Utils.jsm", inforss);
Components.utils.import("chrome://inforss/content/modules/NNTP_Handler.jsm", inforss);

inforss.feed_handlers = inforss.feed_handlers || {};

Components.utils.import("chrome://inforss/content/feed_handlers/factory.jsm", inforss.feed_handlers);

inforss.feed_handlers.factory.register("nntp", inforssFeedNntp);

/* globals inforssFeed */

function inforssFeedNntp(feedXML, manager, menuItem, config)
{
  inforssFeed.call(this, feedXML, manager, menuItem, config);
  return this;
}

inforssFeedNntp.prototype = Object.create(inforssFeed.prototype);
inforssFeedNntp.prototype.constructor = inforssFeedNntp;

Object.assign(inforssFeedNntp.prototype, {

  get_title(item)
  {
    return item.title;
  },

  getCategory(/*item*/)
  {
    return "";
  },

  getDescription(item)
  {
    return item.description;
  },

  //starts the nntp fetch - note once it is finished, we should call
  //this.read_headlines with the array of headlines
  start_fetch()
  {
    const url = this.getUrl();
    const user = this.getUser();
    const nntp = new inforss.NNTP_Handler(
      url, user, this.config.readPassword(url, user));
    nntp.open().then(
      //FIXME I should store the latest article somewhere.
      //Then I could do 'over' from that article, rather than
      //guessing.
      groupinfo => this.read_articles(nntp, groupinfo)
    ).catch(
      e => {
/**/console.log("Error reading feed", url, e)
        this.error = true;
      }
    ).then(
      () => {
        nntp.close();
        this.end_processing();
      }
    );
  },

  read_articles(nntp, group_info)
  {
    const feed_url = "news://" + nntp.host + "/";
    if (group_info.number == 0)
    {
      return;
    }
    //Completely arbitrary limit to articles to fetch.
    const start = group_info.number > 30 ? group_info.hwm - 30 : group_info.lwm;
    const headlines = [];
    const overview = [];
    return nntp.over({ start: start, end: group_info.hwm}).then(
      articles => {
        const promises = [];
        for (let article of articles)
        {
          const headline = {};
          headline.link = feed_url + encodeURIComponent(article[4].slice(1, -1));
          headline.guid = headline.link;
          headline.pubdate = new Date(article[3]);
          article[1] = inforssFeedNntp.decodeQuotedPrintable(article[1]);
          headline.title = "(" + nntp.group + ") " + article[1];
          //Sort of crapness: if we don't already have the headline in the feed,
          //go fetch the body.
          if (this.findHeadline(headline.guid) != null)
          {
            /**/console.log("have headline for", this.getUrl(), headline.guid)
            continue;
          }
          headlines.push(headline);
          overview.push(article);
          promises.push(nntp.fetch_body(article[4]));
        }
        //Oddly, you can get results in over for which the article no longer exists
        return Promise.all(promises.map(p => p.catch(() => undefined)));
      }
      ).then(
        articles => {
          //Because nntp is serial and the promises are kicked off in the order
          //they are placed in the array, we know that articles[n] corresponds to
          //headlines[n]
          const nheadlines = headlines.map(
            (val, index) => {
              const a = val;
              let data = articles[index].join("\n");
              //Should probablly decode this as well.
              data = inforss.htmlFormatConvert(data, false, "text/plain",
                                               "text/html");
              data = data.replace(/^(>>>>.*)$/gm, "<font color='cyan'>$1</font>");
              data = data.replace(/^(> > > >.*)$/gm, "<font color='cyan'>$1</font>");
              data = data.replace(/^(>>>.*)$/gm, "<font color='red'>$1</font>");
              data = data.replace(/^(> > >.*)$/gm, "<font color='red'>$1</font>");
              data = data.replace(/^(>>.*)$/gm, "<font color='green'>$1</font>");
              data = data.replace(/^(> >.*)$/gm, "<font color='green'>$1</font>");
              data = data.replace(/^(>[^>].*)$/gm, "<font color='blue'>$1</font>");
              data = data.replace(/\n/gm, "<br>");
              data =
                "<div style='background-color:#2B60DE; color: white; border-style: solid; border-width:1px; -moz-border-radius: 10px; padding: 6px'><TABLE WIDTH='100%' style='color:white'><TR><TD align='right'><B>From: </B></TD><TD>" +
                overview[index][2] +
                "</TD></TR><TR><TD align='right'><B>Subject: </B></TD><TD>" +
                overview[index][1] +
                "</TD></TR><TR><TD align='right'><B>Date: </B></TD><TD>" +
                overview[index][3] + "</TD></TR></TABLE></div><BR>" +
                data;
              a.description = data;
              return a;
            }).filter(a => a.description != undefined)
            .sort((a, b) => a.pubdate - b.pubdate);
          this.process_headlines(nheadlines);
        }
      ).catch(
        e => inforss.debug(e)
      );
  }
});

//Static methods

//-----------------------------------------------------------------------------------------------------
inforssFeedNntp.decodeQuotedPrintable = function(str)
{
  try
  {
    //Headers have to be RFC2045 encoded (and possible the body as well)
    //look for =?<charset>?[BQ]?text?=
    //This is fairly well broken
    var tmp = str.match(/^(.*)=\?([^\?]*)\?Q\?(.*)\?=(.*)$/);
    if (tmp == null || tmp.length != 5)
    {
      return str;
    }
/**/console.log("Found quote printable", str, tmp)
    return tmp[1] +
        inforssFeedNntp.decodeQuotedPrintableWithCharSet(tmp[3], tmp[2]) +
        tmp[4];
  }
  catch (e)
  {
    inforss.debug(e);
  }
  return null;
};

//-----------------------------------------------------------------------------------------------------
inforssFeedNntp.decodeQuotedPrintableWithCharSet = function(str, charSet)
{
  var returnValue = null;
  var unicodeConverter = Components.classes[
    "@mozilla.org/intl/scriptableunicodeconverter"].createInstance(
    Components.interfaces.nsIScriptableUnicodeConverter);
  unicodeConverter.charset = charSet;
  try
  {
    var tmp = str.match(/^([^=]*)=(..)(.*)$/);
    var code = new Array(1);
    while (tmp != null)
    {
      if (returnValue == null)
      {
        returnValue = unicodeConverter.ConvertToUnicode(tmp[1]) +
          unicodeConverter.Finish();
      }
      else
      {
        returnValue = returnValue + unicodeConverter.ConvertToUnicode(tmp[1]) +
          unicodeConverter.Finish();
      }
      code[0] = parseInt(tmp[2], 16);
      returnValue = returnValue + unicodeConverter.convertFromByteArray(
        code, code.length);
      str = tmp[3];
      tmp = str.match(/^([^=]*)=(..)(.*)$/);
    }
    returnValue = returnValue + unicodeConverter.ConvertToUnicode(str) +
      unicodeConverter.Finish();
  }
  catch (e)
  {
    inforss.debug(e);
  }
  return returnValue;
};
